# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/fastlite',
                'doc_host': 'https://AnswerDotAI.github.io',
                'git_url': 'https://github.com/AnswerDotAI/fastlite',
                'lib_path': 'fastlite'},
  'syms': { 'fastlite.core': { 'fastlite.core.Database.create': ('core.html#database.create', 'fastlite/core.py'),
                               'fastlite.core.Database.q': ('core.html#database.q', 'fastlite/core.py'),
                               'fastlite.core.Database.t': ('core.html#database.t', 'fastlite/core.py'),
                               'fastlite.core.Database.v': ('core.html#database.v', 'fastlite/core.py'),
                               'fastlite.core.Table.__call__': ('core.html#table.__call__', 'fastlite/core.py'),
                               'fastlite.core.Table.__str__': ('core.html#table.__str__', 'fastlite/core.py'),
                               'fastlite.core.Table.c': ('core.html#table.c', 'fastlite/core.py'),
                               'fastlite.core.View.__call__': ('core.html#view.__call__', 'fastlite/core.py'),
                               'fastlite.core.View.__str__': ('core.html#view.__str__', 'fastlite/core.py'),
                               'fastlite.core.View.c': ('core.html#view.c', 'fastlite/core.py'),
                               'fastlite.core._Col': ('core.html#_col', 'fastlite/core.py'),
                               'fastlite.core._Col.__init__': ('core.html#_col.__init__', 'fastlite/core.py'),
                               'fastlite.core._Col.__iter__': ('core.html#_col.__iter__', 'fastlite/core.py'),
                               'fastlite.core._Col.__repr__': ('core.html#_col.__repr__', 'fastlite/core.py'),
                               'fastlite.core._Col.__str__': ('core.html#_col.__str__', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter': ('core.html#_colsgetter', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter.__call__': ('core.html#_colsgetter.__call__', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter.__contains__': ('core.html#_colsgetter.__contains__', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter.__dir__': ('core.html#_colsgetter.__dir__', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter.__getattr__': ('core.html#_colsgetter.__getattr__', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter.__init__': ('core.html#_colsgetter.__init__', 'fastlite/core.py'),
                               'fastlite.core._ColsGetter.__repr__': ('core.html#_colsgetter.__repr__', 'fastlite/core.py'),
                               'fastlite.core._Getter': ('core.html#_getter', 'fastlite/core.py'),
                               'fastlite.core._Getter.__contains__': ('core.html#_getter.__contains__', 'fastlite/core.py'),
                               'fastlite.core._Getter.__getattr__': ('core.html#_getter.__getattr__', 'fastlite/core.py'),
                               'fastlite.core._Getter.__getitem__': ('core.html#_getter.__getitem__', 'fastlite/core.py'),
                               'fastlite.core._Getter.__init__': ('core.html#_getter.__init__', 'fastlite/core.py'),
                               'fastlite.core._Getter.__repr__': ('core.html#_getter.__repr__', 'fastlite/core.py'),
                               'fastlite.core._TablesGetter': ('core.html#_tablesgetter', 'fastlite/core.py'),
                               'fastlite.core._TablesGetter.__dir__': ('core.html#_tablesgetter.__dir__', 'fastlite/core.py'),
                               'fastlite.core._ViewsGetter': ('core.html#_viewsgetter', 'fastlite/core.py'),
                               'fastlite.core._ViewsGetter.__dir__': ('core.html#_viewsgetter.__dir__', 'fastlite/core.py'),
                               'fastlite.core._dataclass': ('core.html#_dataclass', 'fastlite/core.py'),
                               'fastlite.core._edge': ('core.html#_edge', 'fastlite/core.py'),
                               'fastlite.core._get_flds': ('core.html#_get_flds', 'fastlite/core.py'),
                               'fastlite.core._row': ('core.html#_row', 'fastlite/core.py'),
                               'fastlite.core._tnode': ('core.html#_tnode', 'fastlite/core.py'),
                               'fastlite.core.all_dcs': ('core.html#all_dcs', 'fastlite/core.py'),
                               'fastlite.core.create_mod': ('core.html#create_mod', 'fastlite/core.py'),
                               'fastlite.core.diagram': ('core.html#diagram', 'fastlite/core.py')},
            'fastlite.kw': {}}}
